@@ -42,6 +42,7 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         self.setWindowTitle(title)
         self.fs_url = unicode(fs_url)
         self.fs = None
+        self.file_list_items = []
         self.last_index = 0
         self.default_filename = default_filename
         self.file_pattern = file_pattern
@@ -50,17 +51,15 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         self.ui_FileType.setText(self.file_pattern)
         self.browse_folder()
         self.ui_FileType.returnPressed.connect(self.selection_file_type)
-        self.ui_ButtonBox.button(self.ui_ButtonBox.Cancel).clicked.connect(self.cancel)
+        self.ui_SelectedName.textChanged.connect(self.selection_name)
+        self.ui_Cancel.clicked.connect(self.cancel)
         self.action_buttons()
         self.ui_FileList.itemClicked.connect(self.show_name)
+        self.ui_mkdir.clicked.connect(self.make_dir)
 
     def action_buttons(self):
         try:
-            self.ui_ButtonBox.button(self.ui_ButtonBox.Open).clicked.connect(self.action)
-        except AttributeError:
-            pass
-        try:
-            self.ui_ButtonBox.button(self.ui_ButtonBox.Save).clicked.connect(self.action)
+            self.ui_Action.clicked.connect(self.action)
         except AttributeError:
             pass
 
@@ -68,6 +67,8 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         """
         walks through all folders and selects first directory
         """
+        if self.show_save_action:
+            self.ui_Action.setEnabled(True)
         self.ui_DirList.clear()
         self.fs = fs.open_fs(self.fs_url)
         self.ui_DirList.addItem(u'.')
@@ -85,13 +86,25 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         """
         self.last_index = i
         self.selected_dir = self.ui_DirList.currentText()
-        self.ui_FileList.clear()
+        self.ui_FileList.clearContents()
         file_type = self.ui_FileType.text()
+        self.file_list_items = []
+        self.ui_FileList.setColumnCount(1)
+        self.ui_FileList.setRowCount(len(self.file_list_items))
+        self.ui_FileList.verticalHeader().setVisible(False)
+        self.ui_FileList.horizontalHeader().setVisible(False)
+        self.ui_FileList.setShowGrid(False)
+
         for item in sorted(self.fs.listdir(self.selected_dir)):
             if not self.fs.isdir(item) and match_extension(item, [file_type]):
-                self.ui_FileList.addItem(item)
+                self.file_list_items.append(item)
+        self.ui_FileList.setRowCount(len(self.file_list_items))
+        index = 0
+        for item in self.file_list_items:
+            self.ui_FileList.setItem(index, 0,  QtWidgets.QTableWidgetItem(item))
+            index = index + 1
         if self.last_index == 0 and not self.show_save_action:
-            self.ui_FileList.setCurrentRow(0)
+            self.ui_FileList.clearSelection()
             if self.ui_FileList.currentItem() is not None:
                 self.ui_SelectedName.setText(self.ui_FileList.currentItem().text())
 
@@ -100,13 +113,34 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         Action for line edit of file type
         """
         self.selection_directory(self.last_index)
-        self.ui_FileList.setCurrentRow(-1)
+        self.ui_FileList.clearSelection()
         if not self.show_save_action:
             self.ui_SelectedName.setText(None)
 
+    def selection_name(self):
+        self.filename = self.ui_SelectedName.text()
+        if ((self.filename not in (u"",  u".") and self.fs.exists(fs.path.join(self.selected_dir, self.filename)) and
+                        self.filename in self.file_list_items)):
+            self.ui_Action.setEnabled(True)
+            item = self.ui_FileList.findItems(self.filename, QtCore.Qt.MatchExactly)
+            try:
+                self.ui_FileList.selectRow(self.file_list_items.index(self.filename))
+            except TypeError:
+                pass
+        else:
+            if not self.show_save_action:
+                self.ui_Action.setEnabled(False)
+            self.ui_FileList.clearSelection()
+
     def show_name(self):
-        self.filename = self.ui_FileList.item(self.ui_FileList.currentRow()).text()
+        try:
+            self.filename = self.ui_FileList.currentItem().text()
+        except AttributeError:
+            self.filename = None
         self.ui_SelectedName.setText(self.filename)
+        if self.fs.exists(fs.path.join(self.selected_dir, self.filename)):
+            self.ui_Action.setEnabled(True)
+
 
     def cancel(self):
         """
@@ -122,10 +156,23 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         """
         if self.show_save_action:
             self.ui_SelectedName.setEnabled(True)
-            self.ui_ButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel | QtWidgets.QDialogButtonBox.Save)
+            self.ui_Action.setText("Save")
             if self.default_filename is not None:
                 self.ui_SelectedName.setText(self.default_filename)
 
+    def make_dir(self):
+        """
+        shows the make dir dialog und creates a new directory
+        """
+        new_dir, ok = QtWidgets.QInputDialog.getText(self, u"New Folder","Enter a new folder name:",
+                                                  QtWidgets.QLineEdit.Normal, "")
+        if ok:
+            if not u"/" in new_dir and not self.fs.isdir(new_dir):
+                self.fs.makedir(new_dir)
+                self.browse_folder()
+            else:
+                ok = QtWidgets.QMessageBox.warning(self, "New Folder", "Can't create this Folder: {}".format(new_dir))
+
     def action(self):
         """
         Action on open / save button
@@ -145,7 +192,7 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
                 self.close()
         else:
             try:
-                self.filename = self.ui_FileList.item(self.ui_FileList.currentRow()).text()
+                self.filename = self.ui_FileList.currentItem().text()
             except AttributeError:
                 pass
             else:
